@@ -20,7 +20,7 @@ class MarkdownCodeSpans(func.Functor):
     # and contains no markdown code spans.
     # Thus, functions mapping over the text can freely make substitutions,
     # knowing they won't accidentally replace stuff in a code span.
-    def __init__(self, text):
+    def __init__(self, text: str):
         self.__codeSpanReplacements__ = []
         newText = ""
         mode = "text"
@@ -59,18 +59,18 @@ class MarkdownCodeSpans(func.Functor):
 
         super().__init__(newText)
 
-    def map(self, fn):
+    def map(self, fn: t.Callable) -> MarkdownCodeSpans:
         x = MarkdownCodeSpans("")
         x.__val__ = fn(self.__val__)
         x.__codeSpanReplacements__ = self.__codeSpanReplacements__
         return x
 
-    def extract(self):
+    def extract(self) -> str:
         if self.__codeSpanReplacements__:
             # Reverse the list, so I can use pop() to get them starting from the first.
             repls = self.__codeSpanReplacements__[::-1]
 
-            def codeSpanReviver(_):
+            def codeSpanReviver(_: t.Any) -> str:
                 # Match object is the PUA character, which I can ignore.
                 repl = repls.pop()
                 if repl[0] == "" or repl[0].endswith("-"):
@@ -83,10 +83,10 @@ class MarkdownCodeSpans(func.Functor):
                 return f"<code data-opaque data-span-tag={repl[0]} bs-autolink-syntax='{h.escapeAttr(repl[2])}'>{h.escapeHTML(repl[1])}</code>"
 
             return re.sub(r"\ue0ff", codeSpanReviver, self.__val__)
-        return self.__val__
+        return t.cast(str, self.__val__)
 
 
-def stripBOM(doc: t.SpecT):
+def stripBOM(doc: t.SpecT) -> None:
     if len(doc.lines) >= 1 and doc.lines[0].text[0:1] == "\ufeff":
         doc.lines[0].text = doc.lines[0].text[1:]
         m.warn("Your document has a BOM. There's no need for that, please re-save it without a BOM.")
@@ -99,18 +99,18 @@ def fixManualDefTables(doc: t.SpecT) -> None:
     # Def tables generated via datablocks are guaranteed correct,
     # but manually-written ones often don't link up the names in the first row.
     for table in h.findAll("table.propdef, table.descdef, table.elementdef", doc):
-        if h.hasClass(table, "partial"):
+        if h.hasClass(doc, table, "partial"):
             tag = "a"
             attr = "data-link-type"
         else:
             tag = "dfn"
             attr = "data-dfn-type"
-        tag = "a" if h.hasClass(table, "partial") else "dfn"
-        if h.hasClass(table, "propdef"):
+        tag = "a" if h.hasClass(doc, table, "partial") else "dfn"
+        if h.hasClass(doc, table, "propdef"):
             type = "property"
-        elif h.hasClass(table, "descdef"):
+        elif h.hasClass(doc, table, "descdef"):
             type = "descriptor"
-        elif h.hasClass(table, "elementdef"):
+        elif h.hasClass(doc, table, "elementdef"):
             type = "element"
         cell = h.findAll("tr:first-child > :nth-child(2)", table)[0]
         names = [x.strip() for x in h.textContent(cell).split(",")]
@@ -174,14 +174,14 @@ def canonicalizeShortcuts(doc: t.SpecT) -> None:
             el.set("data-dfn-for", _for)
 
 
-def addImplicitAlgorithms(doc: t.SpecT):
+def addImplicitAlgorithms(doc: t.SpecT) -> None:
     # If a container has an empty `algorithm` attribute,
     # but it contains only a single `<dfn>`,
     # assume that the dfn is a description of the algorithm.
     for el in h.findAll("[data-algorithm='']:not(h1):not(h2):not(h3):not(h4):not(h5):not(h6)", doc):
         dfns = h.findAll("dfn", el)
         if len(dfns) == 1:
-            dfnName = config.firstLinkTextFromElement(dfns[0])
+            dfnName = config.firstLinkTextFromElement(dfns[0]) or ""
             el.set("data-algorithm", dfnName)
             dfnFor = dfns[0].get("data-dfn-for")
             if dfnFor:
@@ -199,10 +199,10 @@ def addImplicitAlgorithms(doc: t.SpecT):
 
 
 def checkVarHygiene(doc: t.SpecT) -> None:
-    def isAlgo(el):
+    def isAlgo(el: t.ElementT) -> bool:
         return el.get("data-algorithm") is not None
 
-    def nearestAlgo(el):
+    def nearestAlgo(el: t.ElementT) -> t.ElementT | None:
         # Find the nearest "algorithm" container,
         # either an ancestor with [algorithm] or the nearest heading with same.
         if isAlgo(el):
@@ -213,8 +213,9 @@ def checkVarHygiene(doc: t.SpecT) -> None:
         for heading in h.relevantHeadings(el):
             if isAlgo(heading):
                 return heading
+        return None
 
-    def algoName(el):
+    def algoName(el: t.ElementT) -> str | None:
         # Finds a uniquified algorithm name from an algo container
         algo = nearestAlgo(el)
         if algo is None:
@@ -224,11 +225,11 @@ def checkVarHygiene(doc: t.SpecT) -> None:
         return f"'{algoName}'" + (f" for {algoFor}" if algoFor else "")
 
     # Look for vars that only show up once. These are probably typos.
-    varCounts: t.DefaultDict[t.Tuple[str, str], int] = defaultdict(lambda: 0)
+    varCounts: defaultdict[tuple[str, str | None], int] = defaultdict(lambda: 0)
     for el in h.findAll("var:not([data-var-ignore])", doc):
         key = (h.foldWhitespace(h.textContent(el)).strip(), algoName(el))
         varCounts[key] += 1
-    foldedVarCounts: t.DefaultDict[t.Tuple[str, str], int] = defaultdict(lambda: 0)
+    foldedVarCounts: defaultdict[tuple[str, str | None], int] = defaultdict(lambda: 0)
     atLeastOneAlgo = False
     for (var, algo), count in varCounts.items():
         if algo:
@@ -497,10 +498,10 @@ def processDfns(doc: t.SpecT) -> None:
     dfns = h.findAll(config.dfnElementsSelector, doc)
     classifyDfns(doc, dfns)
     h.fixupIDs(doc, dfns)
-    doc.refs.addLocalDfns(dfn for dfn in dfns if dfn.get("id") is not None)
+    doc.refs.addLocalDfns(doc, (dfn for dfn in dfns if dfn.get("id") is not None))
 
 
-def determineDfnType(dfn: t.ElementT, inferCSS=False) -> str:
+def determineDfnType(doc: t.SpecT, dfn: t.ElementT, inferCSS: bool = False) -> str:
     # 1. Look at data-dfn-type
     if dfn.get("data-dfn-type"):
         return t.cast(str, dfn.get("data-dfn-type"))
@@ -515,9 +516,9 @@ def determineDfnType(dfn: t.ElementT, inferCSS=False) -> str:
         if ancestor.get("data-dfn-type"):
             return t.cast(str, ancestor.get("data-dfn-type"))
         for cls, type in config.dfnClassToType.items():
-            if h.hasClass(ancestor, cls):
+            if h.hasClass(doc, ancestor, cls):
                 return type
-            if h.hasClass(ancestor, "idl") and not h.hasClass(ancestor, "extract"):
+            if h.hasClass(doc, ancestor, "idl") and not h.hasClass(doc, ancestor, "extract"):
                 return "interface"
     # 4. Introspect on the text
     if inferCSS:
@@ -541,10 +542,10 @@ def determineDfnType(dfn: t.ElementT, inferCSS=False) -> str:
     return "dfn"
 
 
-def classifyDfns(doc: t.SpecT, dfns: t.List[t.ElementT]) -> None:
+def classifyDfns(doc: t.SpecT, dfns: list[t.ElementT]) -> None:
     dfnTypeToPrefix = {v: k for k, v in config.dfnClassToType.items()}
     for el in dfns:
-        dfnType = determineDfnType(el, inferCSS=doc.md.inferCSSDfns)
+        dfnType = determineDfnType(doc, el, inferCSS=doc.md.inferCSSDfns)
         if dfnType not in config.dfnTypes:
             m.die(f"Unknown dfn type '{dfnType}':\n{h.outerHTML(el)}", el=el)
             continue
@@ -721,10 +722,10 @@ def determineLinkType(el: t.ElementT) -> str:
 
 
 def determineLinkText(el: t.ElementT) -> str:
-    linkType = el.get("data-link-type")
+    linkType = el.get("data-link-type", "")
     contents = h.textContent(el)
     if el.get("data-lt"):
-        linkText = t.cast(str, el.get("data-lt"))
+        linkText = el.get("data-lt", "")
     elif config.linkTypeIn(linkType, "function") and re.match(r"^[\w-]+\(.*\)$", contents):
         # Remove arguments from CSS function autolinks,
         # as they should always be defined argument-less
@@ -908,13 +909,10 @@ def processAutolinks(doc: t.SpecT) -> None:
             error=not okayToFail and not ignorable,
         )
         # Capture the reference (and ensure we add a biblio entry) if it
-        # points to an external specification. We check the spec name here
-        # rather than checking `status == "local"`, as "local" refs include
-        # those defined in `<pre class="anchor">` datablocks, which we do
-        # want to capture here.
-        if ref and ref.spec and doc.refs.spec and ref.spec.upper() != doc.refs.spec.upper():
+        # points to an external specification.
+        if ref and ref.status != "local":
             spec = ref.spec.upper()
-            key = ref.for_[0] if ref.for_ else ""
+            forVal = ref.for_[0] if ref.for_ else None
 
             # If the ref is from an anchor block, it knows what it's doing.
             # Don't follow obsoletion chains.
@@ -930,7 +928,7 @@ def processAutolinks(doc: t.SpecT) -> None:
             if biblioRef:
                 spec = biblioRef.linkText.upper()
                 registerBiblioUsage(doc, biblioRef, el=el)
-            doc.externalRefsUsed[spec][ref.text][key] = ref
+            doc.externalRefsUsed.addRef(spec, ref, forVal)
 
         if ref:
             el.set("href", ref.url)
@@ -946,9 +944,9 @@ def processAutolinks(doc: t.SpecT) -> None:
     h.dedupIDs(doc)
 
 
-def registerBiblioUsage(doc: t.SpecT, ref: biblio.BiblioEntry, el: t.ElementT, type: t.Optional[str] = None) -> None:
+def registerBiblioUsage(doc: t.SpecT, ref: biblio.BiblioEntry, el: t.ElementT, type: str | None = None) -> None:
     if type is None:
-        if h.isNormative(el, doc):
+        if h.isNormative(doc, el):
             type = "normative"
         else:
             type = "informative"
@@ -961,7 +959,7 @@ def registerBiblioUsage(doc: t.SpecT, ref: biblio.BiblioEntry, el: t.ElementT, t
         return
     spec = ref.linkText.upper()
     biblioStorage[spec] = ref
-    doc.externalRefsUsed[spec]["_biblio"] = ref
+    doc.externalRefsUsed.addBiblio(spec, ref)
 
 
 def decorateAutolink(doc: t.SpecT, el: t.ElementT, linkType: str, linkText: str, ref: refs.RefWrapper) -> None:
@@ -982,6 +980,10 @@ def decorateAutolink(doc: t.SpecT, el: t.ElementT, linkType: str, linkText: str,
         if linkText in doc.typeExpansions:
             titleText = doc.typeExpansions[linkText]
         else:
+            if linkText == "":
+                print(h.outerHTML(el, literal=True))
+                assert linkText != ""
+
             typeRefs = doc.refs.queryAllRefs(linkFor=linkText, ignoreObsoletes=True)
             texts = sorted({ref.text for ref in typeRefs})
             if typeRefs:
@@ -996,7 +998,7 @@ def removeMultipleLinks(doc: t.SpecT) -> None:
     # only keep the first.
     if not doc.md.removeMultipleLinks:
         return
-    paras: t.DefaultDict[t.Any, t.DefaultDict[str, t.List[t.Any]]]
+    paras: defaultdict[t.ElementT | None, defaultdict[str, list[t.ElementT]]]
     paras = defaultdict(lambda: defaultdict(list))
     for el in h.findAll("a[data-link-type]", doc):
         if h.hasAncestor(el, lambda x: x.tag in ["pre", "xmp"]):
@@ -1055,7 +1057,7 @@ def addSelfLinks(doc: t.SpecT) -> None:
     if doc.md.slimBuildArtifact:
         return
 
-    def makeSelfLink(el):
+    def makeSelfLink(el: t.ElementT) -> t.ElementT:
         return h.E.a({"href": "#" + h.escapeUrlFrag(el.get("id", "")), "class": "self-link"})
 
     dfnElements = h.findAll(config.dfnElementsSelector, doc)
@@ -1132,8 +1134,8 @@ def cleanupHTML(doc: t.SpecT) -> None:
             el.tag = "{http://www.w3.org/2000/svg}tspan"
 
         # Add .algorithm to [algorithm] elements, for styling
-        if el.get("data-algorithm") is not None and not h.hasClass(el, "algorithm"):
-            h.addClass(el, "algorithm")
+        if el.get("data-algorithm") is not None and not h.hasClass(doc, el, "algorithm"):
+            h.addClass(doc, el, "algorithm")
 
         # Allow MD-generated lists to be surrounded by HTML list containers,
         # so you can add classes/etc without an extraneous wrapper.
@@ -1177,22 +1179,22 @@ def cleanupHTML(doc: t.SpecT) -> None:
             h.removeAttr(el, "data-md")
 
         # Mark pre.idl blocks as .def, for styling
-        if el.tag == "pre" and h.hasClass(el, "idl") and not h.hasClass(el, "def"):
-            h.addClass(el, "def")
+        if el.tag == "pre" and h.hasClass(doc, el, "idl") and not h.hasClass(doc, el, "def"):
+            h.addClass(doc, el, "def")
 
         # Tag classes on wide types of dfns/links
         if el.tag in config.dfnElements:
             if el.get("data-dfn-type") in config.idlTypes:
-                h.addClass(el, "idl-code")
+                h.addClass(doc, el, "idl-code")
             if el.get("data-dfn-type") in config.maybeTypes.union(config.linkTypeToDfnType["propdesc"]):
                 if not h.hasAncestor(el, lambda x: x.tag == "pre"):
-                    h.addClass(el, "css")
+                    h.addClass(doc, el, "css")
         if el.tag == "a":
             if el.get("data-link-type") in config.idlTypes:
-                h.addClass(el, "idl-code")
+                h.addClass(doc, el, "idl-code")
             if el.get("data-link-type") in config.maybeTypes.union(config.linkTypeToDfnType["propdesc"]):
                 if not h.hasAncestor(el, lambda x: x.tag == "pre"):
-                    h.addClass(el, "css")
+                    h.addClass(doc, el, "css")
 
         # Remove duplicate linking texts.
         if (
@@ -1206,7 +1208,7 @@ def cleanupHTML(doc: t.SpecT) -> None:
         # (Used when the ''foo'' shorthand doesn't work.)
         if el.tag == "css":
             el.tag = "span"
-            h.addClass(el, "css")
+            h.addClass(doc, el, "css")
 
         # Assert IDs are used to tag arbitrary sections with an ID so you can point tests at it.
         # (The ID will be guaranteed stable across publications, but guaranteed to change when the text changes.)
@@ -1221,7 +1223,7 @@ def cleanupHTML(doc: t.SpecT) -> None:
             el.set("id", h.safeID(doc, "assert-" + h.hashContents(el)))
 
         # Add ARIA role of "note" to class="note" elements
-        if el.tag in ["div", "p"] and h.hasClass(el, doc.md.noteClass):
+        if el.tag in ["div", "p"] and h.hasClass(doc, el, doc.md.noteClass):
             el.set("role", "note")
 
         # Look for nested <a> elements, and warn about them.
@@ -1237,13 +1239,13 @@ def cleanupHTML(doc: t.SpecT) -> None:
                 if letter.isalpha() and letter.islower():
                     break
             else:
-                h.addClass(el, "allcaps")
+                h.addClass(doc, el, "allcaps")
 
         # If a markdown-generated <dt> contains only a single paragraph,
         # remove that paragraph so it just contains naked text.
         if el.tag == "dt" and el.get("data-md") is not None:
-            child = h.hasOnlyChild(el)
-            if child is not None and child.tag == "p" and h.emptyText(el.text) and h.emptyText(child.tail):
+            dtChild = h.hasOnlyChild(el)
+            if dtChild is not None and h.tagName(dtChild) == "p" and h.emptyText(el.text) and h.emptyText(dtChild.tail):
                 flattenEls.append(el)
 
         # Remove a bunch of attributes
@@ -1304,7 +1306,7 @@ def cleanupHTML(doc: t.SpecT) -> None:
             # Remove *all* data- attributes.
             for attrName in el.attrib:
                 if str(attrName).startswith("data-"):
-                    h.removeAttr(el, attrName)
+                    h.removeAttr(el, str(attrName))
     if head is not None:
         for el in strayHeadEls:
             head.append(el)
@@ -1324,7 +1326,7 @@ def finalHackyCleanup(text: str) -> str:
     return text
 
 
-def hackyLineNumbers(lines: t.List[Line]) -> t.List[Line]:
+def hackyLineNumbers(lines: list[Line]) -> list[Line]:
     # Hackily adds line-number information to each thing that looks like an open tag.
     # This is just regex text-munging, so potentially dangerous!
     for line in lines:
@@ -1340,7 +1342,7 @@ def correctFrontMatter(doc: t.SpecT) -> None:
     # Detect and move around some bits of information,
     # if you provided them in your
     # If you provided an <h1> manually, use that element rather than whatever the boilerplate contains.
-    h1s = [h1 for h1 in h.findAll("h1", doc) if h.isNormative(h1, doc)]
+    h1s = [h1 for h1 in h.findAll("h1", doc) if h.isNormative(doc, h1)]
     if len(h1s) == 2:
         h.replaceNode(h1s[0], h1s[1])
 
@@ -1522,7 +1524,7 @@ def inlineRemoteIssues(doc: t.SpecT) -> None:
                 ),
                 h.E.a({"href": href}, data["title"]),
             )
-            h.addClass(el, "no-marker")
+            h.addClass(doc, el, "no-marker")
         else:
             h.appendChild(
                 el,
@@ -1532,7 +1534,7 @@ def inlineRemoteIssues(doc: t.SpecT) -> None:
                 ),
                 *h.parseHTML(data["body_html"]),
             )
-            h.addClass(el, "no-marker")
+            h.addClass(doc, el, "no-marker")
         if el.tag == "p":
             el.tag = "div"
     # Save the cache for later
@@ -1547,12 +1549,12 @@ def inlineRemoteIssues(doc: t.SpecT) -> None:
 def addNoteHeaders(doc: t.SpecT) -> None:
     # Finds <foo heading="bar"> and turns it into a marker-heading
     for el in h.findAll("[heading]", doc):
-        h.addClass(el, "no-marker")
-        if h.hasClass(el, "note"):
+        h.addClass(doc, el, "no-marker")
+        if h.hasClass(doc, el, "note"):
             preText = "NOTE: "
-        elif h.hasClass(el, "issue"):
+        elif h.hasClass(doc, el, "issue"):
             preText = "ISSUE: "
-        elif h.hasClass(el, "example"):
+        elif h.hasClass(doc, el, "example"):
             preText = "EXAMPLE: "
         else:
             preText = ""
@@ -1654,16 +1656,16 @@ def addImageSize(doc: t.SpecT) -> None:
             )
 
 
-def processIDL(doc) -> None:
+def processIDL(doc: t.SpecT) -> None:
     localDfns = set()
     for pre in h.findAll("pre.idl, xmp.idl", doc):
         if pre.get("data-no-idl") is not None:
             continue
-        if not h.isNormative(pre, doc):
+        if not h.isNormative(doc, pre):
             continue
         localDfns.update(idl.markupIDLBlock(pre, doc))
 
     dfns = h.findAll("pre.idl:not([data-no-idl]) dfn, xmp.idl:not([data-no-idl]) dfn", doc) + list(localDfns)
     classifyDfns(doc, dfns)
     h.fixupIDs(doc, dfns)
-    doc.refs.addLocalDfns(dfn for dfn in dfns if dfn.get("id") is not None)
+    doc.refs.addLocalDfns(doc, (dfn for dfn in dfns if dfn.get("id") is not None))
